local AceAddon = AceLibrary("AceAddon-2.0")
local Tablet = AceLibrary("Tablet-2.0")
local Tooltip = AceLibrary("Gratuity-2.0")

local L = AceLibrary("AceLocale-2.2"):new("Cartographer_Trainers")
L:RegisterTranslations("enUS", function() return {
	["Filter"] = true,
	["Filter out trainers"] = true,
  
    ["Trainers"] = true,
    ["Module description"] = "Trainer Mapnotes for Cartographer",
  
	["Select all"] = true,
	["Select none"] = true,
    
    ["Icon alpha"] = true,
	["Alpha transparency of the icon"] = true,
	["Icon size"] = true,
	["Size of the icons on the map"] = true,  
    ["Show minimap icons"] = true,
    ["Show trainer icons on the minimap"] = true,
    ["Import"] = true,
    ["Import Trainers"] = true,
    
    ["Trainer"] = true,
    ["Mage Trainer"] = true,
} end)

L:RegisterTranslations("koKR", function() return {
	["Filter"] = "필터",
	["Filter out trainers"] = "상급 기술자 아웃 필터",

    ["Trainers"] = "상급 기술자",
    ["Module description"] = "맵노트에 상급 기술자를 표시합니다.",

    ["Select all"] = "모두 선택",
    ["Select none"] = "선택 없음",

	["Icon alpha"] = "아이콘 투명도",
	["Alpha transparency of the icon"] = "아이콘의 투명도를 조절합니다.",
	["Icon size"] = "아이콘 크기",
	["Size of the icons on the map"] = "아이콘의 크기를 조절합니다.",
    ["Show minimap icons"] = "미니맵 아이콘 보기",
    ["Show trainer icons on the minimap"] = "미니맵에 상급자 아이콘을 표시합니다.",
    ["Import"] = "입력하기",
    ["Import Trainers"] = "기술자 정보를 입력합니다.",

    ["Trainer"] = "상급 기술자",
    ["Mage Trainer"] = "마법사 상급자",
} end)

L:RegisterTranslations("deDE", function() return {
	["Filter"] = "Filter",
	["Filter out trainers"] = "Lehrer filtern",
	
    ["Trainers"] = "Lehrer",
    ["Module description"] = "Lehrer-Kartennotizen f\195\188r Cartographer",

    ["Select all"] = "Alles ausw\195\164hlen",
    ["Select none"] = "Nichts ausw\195\164hlen",
    
    ["Icon alpha"] = "Transparenz",
    ["Alpha transparency of the icon"] = "Transparenz der Symbole",
    ["Icon size"] = "Symbolgr\195\182\195\159e",
    ["Size of the icons on the map"] = "Gr\195\182\195\159e des Symbols auf der Karte",
    ["Import"] = true,
    ["Import Trainers"] = true,

    ["Trainer"] = "Lehrer",
} end)

L:RegisterTranslations("zhTW", function() return {
	["Filter"] = "篩檢",
	["Filter out trainers"] = "篩檢訓練師",
  
    ["Trainers"] = "訓練師",
    ["Module description"] = "Cartographer 訓練師地圖註記",

    ["Select all"] = "全部選擇",
    ["Select none"] = "全部取消",
    ["Icon alpha"] = true,
	["Alpha transparency of the icon"] = true,
	["Icon size"] = true,
	["Size of the icons on the map"] = true,  
    ["Show minimap icons"] = true,
    ["Show trainer icons on the minimap"] = true,
    ["Import"] = true,
    ["Import Trainers"] = true,

    ["Trainer"] = "訓練師",
} end)

L:RegisterTranslations("esES", function() return {
	["Filter"] = "Filtrar",
	["Filter out trainers"] = "Filtra instructores",
  
    ["Trainers"] = "Instructores",
    ["Module description"] = "Notas de mapa de instructores para Cartographer",
  
	["Select all"] = "Seleccionar todo",
	["Select none"] = "No seleccionar nada",

    ["Icon alpha"] = "Transparencia de icono",
    ["Alpha transparency of the icon"] = "Transparencia del icono",
    ["Icon size"] = "Tama\195\177o del icono",
    ["Size of the icons on the map"] = "Tama\195\177o de los iconos en el mapa",
    ["Import"] = true,
    ["Import Trainers"] = true,

    ["Trainer"] = "Instructor",
} end)

local mod
if AceLibrary:HasInstance("AceDebug-2.0") then 
    mod = Cartographer:NewModule("Trainers", "AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0", "AceDebug-2.0")
    mod:SetDebugging(false)
else
    mod = Cartographer:NewModule("Trainers", "AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0")
    -- mod.Debug = function() return end
end

mod:RegisterDB("Cartographer_TrainersDB")

mod.filters = {
    ["Trainer"] = L["Trainer"],
    ["Mage Trainer"] = L["Mage Trainer"],
}

mod.icons = {
	["Trainer"] = {
		text = L["Trainer"],
		path = "Interface\\AddOns\\Cartographer_Trainers\\Artwork\\INV_Misc_Book_03",
		width = 14,
		height = 14
	},
    ["Mage Trainer"] = {
        text = L["Mage Trainer"],
        path = "Interface\\AddOns\\Cartographer_Trainers\\Artwork\\INV_Misc_Book_03",
        width = 14,
        height = 14,
    },    
}

mod.trainers = {}

function mod:OnInitialize()
	self:RegisterDefaults("profile", {
		filters = {
			['*'] = true,
		},
        iconAlpha = 1,
		iconScale = 1,   
        minimapIcons = true,
	})

	local aceopts = {}
	aceopts.toggle = {
		name = AceLibrary("AceLocale-2.2"):new("Cartographer")["Enabled"],
		desc = AceLibrary("AceLocale-2.2"):new("Cartographer")["Suspend/resume this module."],
		type  = 'toggle',
		order = -1,
		get   = function() return Cartographer:IsModuleActive(self) end,
		set   = function() Cartographer:ToggleModuleActive(self) end,
	}
    
	aceopts.alpha = {
		name = L["Icon alpha"],
		desc = L["Alpha transparency of the icon"],
		type = 'range',
		min = 0.1,
		max = 1,
		step = 0.05,
		isPercent = true,
		get = "GetIconAlpha",
		set = "SetIconAlpha",
		order = 1
	}
	aceopts.scale = {
		name = L["Icon size"],
		desc = L["Size of the icons on the map"],
		type = 'range',
		min = 0.5,
		max = 2,
		step = 0.05,
		isPercent = true,
		get = "GetIconScale",
		set = "SetIconScale",
		order = 2
	}      
    aceopts.minimapicons = {
		name = L["Show minimap icons"],
		desc = L["Show trainer icons on the minimap"],
		type = 'toggle',
		set = "ToggleShowingMinimapIcons",
		get = "IsShowingMinimapIcons",
		order = 3,
	}    

	aceopts.filter = {
		name = L["Filter"],
		desc = L["Filter out trainers"],
		type = 'group',
		args = {
			all = {
				name = L["Select all"],
				desc = L["Select all"],
				type = 'execute',
				func = function()
					for k in pairs(self.filters) do
						self:ToggleShowingTrainers(k, true)
					end
				end,
				order = 1,
			},
			none = {
				name = L["Select none"],
				desc = L["Select none"],
				type = 'execute',
				func = function()
					for k in pairs(self.filters) do
						self:ToggleShowingTrainers(k, false)
					end
				end,
				order = 2,
			},
			blank = {
				type = 'header',
				order = 3,
			}
		},
        order = 4,
	}
    
    aceopts.import = {
        name = L["Import"],
        desc = L["Import Trainers"],
        type = 'execute',
        func = 'ImportTrainers',
        order = 5,
        hidden = true,
    }    

   	Cartographer.options.args.Trainers = {
		name = L["Trainers"],
		desc = L["Module description"],
		type = 'group',
		args = aceopts,
		handler = self,
	}
	-- AceLibrary("AceConsole-2.0"):InjectAceOptionsTable(self, Cartographer.options.args.Trainers)

	for k,v in pairs(self.filters) do
		local k = k
		aceopts.filter.args[k] = {
			name = v,
			desc = v,
			type = 'toggle',
			get = function()
				return self:IsShowingTrainers(k)
			end,
			set = function(value)
				return self:ToggleShowingTrainers(k, value)
			end,
		}
	end    
    
    if (not self.db.faction.notes) then self.db.faction.notes = {} end
    if (not mod.trainers) and (self.db.server.trainers ~= nil) then mod.trainers = self.db.server.trainers end
end

function mod:OnEnable()
	if Cartographer_Notes then
		if not self.iconsregistered then
			for k,v in pairs(self.icons) do
				Cartographer_Notes:RegisterIcon(k, v)
			end
			self.iconsregistered = true
		end

		Cartographer_Notes:RegisterNotesDatabase('Trainers', self.db.faction.notes, self)
	else
		Cartographer:ToggleModuleActive(self, false)
	end
    
    -- if (School) or (TrainerFu) then
    if (SchoolNotes) or (TrainerFu_TrainerNotes) then
        Cartographer:ToggleModuleActive(self, false)
    end
    
    if (Cartographer:IsModuleActive(self)) then
        self:RegisterEvent("TRAINER_SHOW", function() self:RegisterTrainer(UnitName("NPC")) end, 2)
    end
end

function mod:OnDisable()
	self:UnregisterAllEvents()

	if Cartographer_Notes then
		Cartographer_Notes:UnregisterNotesDatabase('Trainers')
	end
end

function mod:SetNote(trainer)
	local x, y = GetPlayerMapPosition("player")
	if x == 0 and y == 0 then return end
	local zone = GetRealZoneText()
    
    local note = {}
    note.title = trainer.name
    note.icon = "Trainer"
    note.info = "Trainer"

    if (trainer.type) then
        if (mod.filters[trainer.type] and mod.icons[trainer.type]) then note.icon = trainer.type end
        note.info = trainer.type
    end    
    
	for zone, x_p, y_p, icon, creator, data in Cartographer_Notes:IterateNearbyNotes(zone, x, y, 10, "Trainers", nil, true) do
		if data.title == note.title then
			Cartographer_Notes:DeleteNote(zone, x_p, y_p)
		end
	end
	Cartographer_Notes:SetNote(zone, x, y, note.icon, "Trainers", note)
end

function mod:RegisterTrainer(trainer)
    if (not trainer) then return end
    if (type(trainer) ~= "string") then return end

    local zone = GetRealZoneText()
    local x,y = GetPlayerMapPosition("player")
    
    if (not mod.trainers) then mod.trainers = {} end
    if (not mod.trainers[zone]) then mod.trainers[zone] = {} end
    if (not mod.trainers[zone][trainer]) then
        mod.trainers[zone][trainer] = {}
        mod.trainers[zone][trainer].name = trainer
        mod.trainers[zone][trainer].x = x
        mod.trainers[zone][trainer].y = y
        Tooltip:SetUnit("npc")
        local result = Tooltip:GetLine(2)
        if (result) then mod.trainers[zone][trainer].type = result end
        mod.trainers[zone][trainer].faction = UnitFactionGroup("player")
        
        self.db.server.trainers = mod.trainers
    end

    local entry = mod.trainers[zone][trainer]
    entry.zone = zone
    
    self:SetNote(entry)
end

function mod:ImportTrainers()
end

function mod:OnNoteTooltipRequest(zone, id, trainer_data, minimap)
    if (type(trainer_data) == "table") then
        Tablet:SetTitle(trainer_data.title)
        if (trainer_data.info) and (trainer_data.info:len() > 0) and (not minimap) then
            local trainerInfo = trainer_data.info
            if (L:HasTranslation(trainer_data.info)) then
                trainerInfo = L[trainerInfo]
            end
            Tablet:AddCategory(
                'columns', 2,
                'hideBlankLine', true
            ):AddLine(
               'text', trainerInfo,
               'textR', 255,
               'textG', 255,
               'textB', 255,
               'wrap', true               
            )
        end
        if (trainer_data.info2) and (trainer_data.info2:len() > 0) and (not minimap) then
            local trainerInfo2 = trainer_data.info
            if (L:HasTranslation(trainer_data.info2)) then
                trainerInfo2 = L[trainerInfo2]
            end
            Tablet:AddCategory(
                'columns', 2,
                'hideBlankLine', true
            ):AddLine(
                'text', trainerInfo2,
                'textR', 255,
                'textG', 255,
                'textB', 255,
                'wrap', true
            )
        end
    else
        Tablet:SetTitle(trainer_data)
    end

	Tablet:SetTitleColor(0, 0.8, 0)	

	if Cartographer_Notes:IsShowingCreator() and not minimap then
		Tablet:AddCategory(
			'columns', 2,
			'hideBlankLine', true
		):AddLine(
			'text', AceLibrary("AceLocale-2.2"):new("Cartographer-Notes")["Created by"],
			'text2', L["Trainers"]
		)
	end
end

function mod:IsNoteHidden(zone, id, trainer)
	if type(trainer) == "table" then
		return not self.db.profile.filters[trainer.icon]
	else
		return not self.db.profile.filters[trainer]
	end
end

function mod:IsShowingTrainers(trainertype)
	return self.db.profile.filters[trainertype]
end

function mod:ToggleShowingTrainers(trainertype, value)
	if value == nil then
		value = not self.db.profile.filters[trainertype]
	end
	self.db.profile.filters[trainertype] = value

	Cartographer_Notes:RefreshMap()
end

function mod:IsTracking(zone,id,data)
    return true
end

function mod:IsShowingMinimapIcons()
	return self.db.profile.minimapIcons
end

function mod:ToggleShowingMinimapIcons()
	self.db.profile.minimapIcons = not self.db.profile.minimapIcons
end

function mod:IsMiniNoteHidden(zone,id,data)
	return not self.db.profile.minimapIcons
end

function mod:GetIconScale()
	return self.db.profile.iconScale
end

function mod:SetIconScale(value)
	self.db.profile.iconScale = value
	Cartographer_Notes:RefreshMap()
end
function mod:GetIconAlpha()
	return self.db.profile.iconAlpha
end

function mod:SetIconAlpha(value)
	self.db.profile.iconAlpha = value
	Cartographer_Notes:RefreshMap()
end

function mod:GetNoteTransparency(zone,id,data)
	return self.db.profile.iconAlpha
end

function mod:GetNoteScaling(zone,id,data)
	return self.db.profile.iconScale
end
